/* 
**
** A simple screensaver that execute a script when starts and when stops.
** Version: 1.0
** Written by Andrea Anzani
** freely base on OpenBeOS ScreenSaver by Marcus Overhagen. All Rights Reserved.
** This file may be used under the terms of the OpenBeOS License.
*/

#include <ScreenSaver.h>
#include <View.h>
#include <StringView.h>
#include <Font.h>
#include <stdlib.h>
#include <String.h>
#include <TextControl.h>


BString 	fText;
BString		fStart;
BString		fStop;
bool			clear;

class SetupView : public BView
{
	
public:
	SetupView(BRect frame, const char *name)
	 : BView(frame, name, 0, B_FOLLOW_ALL)
	{
	}

	void AttachedToWindow()
	{
		BMessage *msg;
		SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
		BRect r(10, 5, 230, 25);
		AddChild(new BStringView(r, B_EMPTY_STRING, "ScriptScreenSaver by Andrea Anzani"));
		r.OffsetBy(0,25);
		AddChild(new BStringView(r, B_EMPTY_STRING, "freely based on:"));
		r.OffsetBy(0,15);
		AddChild(new BStringView(r, B_EMPTY_STRING, " OpenBeOS ScreenSaver by Marcus Overhagen"));
		
		r.OffsetBy(0,35);
		AddChild(new BStringView(r, B_EMPTY_STRING, "Text:"));
		msg=new BMessage('text');
		r.OffsetBy(0,25);
		AddChild(_text=new BTextControl(r, B_EMPTY_STRING,NULL,fText.String(),msg));
		_text->SetTarget(this);
		
		//start
		r.OffsetBy(0,35);
		AddChild(new BStringView(r, B_EMPTY_STRING, "Start script:"));
		msg=new BMessage('star');
		r.OffsetBy(0,25);
		AddChild(_start=new BTextControl(r, B_EMPTY_STRING,NULL,fStart.String(),msg));
		_start->SetTarget(this);
		//stop
		r.OffsetBy(0,35);
		AddChild(new BStringView(r, B_EMPTY_STRING, "Stop script:"));
		msg=new BMessage('stop');
		r.OffsetBy(0,25);
		AddChild(_stop=new BTextControl(r, B_EMPTY_STRING,NULL,fStop.String(),msg));
		_stop->SetTarget(this);

	}

	void MessageReceived(BMessage *msg)
	{

		switch(msg->what)
		{
			case 'text' :
				clear=true;
				fText.SetTo(_text->Text());	
			break;
			case 'star' :
				fStart.SetTo(_start->Text());	
			break;
			case 'stop' :
				fStop.SetTo(_stop->Text());	
			break;
			default :
				BView::MessageReceived(msg);
			break;
		}
	}
	
	private:
		BTextControl*	_text;
		BTextControl*	_start;
		BTextControl*	_stop;
};


class ScreenSaver : public BScreenSaver
{
public:
	ScreenSaver(BMessage *archive, image_id);
	void Draw(BView *view, int32 frame);
	void StartConfig(BView *view);
	status_t StartSaver(BView *view, bool preview);
	void		 StopSaver();
	status_t SaveState(BMessage *into) const;
private:
	//const char *fText;
	float fX;
	float fY;
	float fSizeX;
	float fSizeY;
	float fTextHeight;
	float fTextWith;
	bool fIsPreview;
	
	void	Execute(const char*);
};

BScreenSaver *instantiate_screen_saver(BMessage *msg, image_id image) 
{ 
	return new ScreenSaver(msg, image);
} 

ScreenSaver::ScreenSaver(BMessage *archive, image_id id) :
	BScreenSaver(archive, id),
	fX(0),
	fY(0)
{
	clear=true;
	//fText.SetTo("OpenBeOS!");
	
	archive->FindString("text",&fText);
	if(fText=="") fText.SetTo("OpenBeOS!");
	archive->FindString("start",&fStart);
	archive->FindString("stop",&fStop);
	
}

status_t 
ScreenSaver::SaveState(BMessage *into) const
{
	into->AddString("text",fText);
	into->AddString("start",fStart);
	into->AddString("stop",fStop);
	
	return B_OK;
}

void 
ScreenSaver::StartConfig(BView *view) 
{ 
	view->AddChild(new SetupView(view->Bounds(),""));
} 

status_t 
ScreenSaver::StartSaver(BView *view, bool preview)
{
	// save view dimensions and preview mode
	fIsPreview = preview;
	fSizeX = view->Bounds().Width();
	fSizeY = view->Bounds().Height();
	
	// set a new font, about 1/8th of view height, and bold
	BFont font;
	view->GetFont(&font);
	font.SetSize(fSizeY / 8);
	font.SetFace(B_BOLD_FACE);
	view->SetFont(&font);
	
	// find out space needed for text display
	BRect rect;
	escapement_delta delta;
	delta.nonspace = 0;
	delta.space = 0;
	const char* x=fText.String();
	font.GetBoundingBoxesForStrings(&(x),1,B_SCREEN_METRIC,&delta,&rect);
	fTextHeight = rect.Height();
	fTextWith = rect.Width();
	
	// seed the random number generator
	srand((int)system_time());
	
	Execute(fStart.String());
	
	return B_OK;
}

void
ScreenSaver::StopSaver()
{
	Execute(fStop.String());	
}

void 
ScreenSaver::Draw(BView *view, int32 frame)
{
	if (frame == 0 || clear) { 
		// fill with black on first frame
		view->SetLowColor(0, 0, 0); 
		view->FillRect(view->Bounds(), B_SOLID_LOW); 
		clear=false;
	} else {
		// erase old text on all other frames
		view->SetHighColor(0,0,0);
		view->DrawString(fText.String(),BPoint(fX,fY));
	}

	// find some new text coordinates
	fX = rand() % int(fSizeX - fTextWith);
	fY = rand() % int(fSizeY - fTextHeight - (fIsPreview ? 2 : 20)) + fTextHeight;

	// draw new text
	view->SetHighColor(0,255,0);
	view->DrawString(fText.String(),BPoint(fX,fY));

	// randomize time until next update (preview mode is faster)
	SetTickSize(((rand() % 4) + 1) * (fIsPreview ? 300000 : 1000000));
}

void
ScreenSaver::Execute(const char* command)
{
	int32  argc  =  1;
	const char ** argv =	 new const char * [argc + 1];
	
	argv[0]	=	strdup(command);
	argv[1]	=	NULL;
	
	// "load" command. 
    thread_id ret  =  load_image(argc, argv, (const char **) environ); 
    // resume
    resume_thread(ret);	
}